//+------------------------------------------------------------------+
//|                                            FridayCloseExpert.mq5 |
//|                                        Copyright  2018, Amr Ali |
//|                             https://www.mql5.com/en/users/amrali |
//+------------------------------------------------------------------+
#property copyright "Copyright  2018, Amr Ali"
#property link      "https://www.mql5.com/en/users/amrali"
#property version   "1.000"
#property description "project description."
//--- Includes
#include <Trade\Trade.mqh>
#include <Trade\PositionInfo.mqh>
//---
CTrade         m_trade;      // trading object
CPositionInfo  m_position;   // position info object
//--- Input parameters                                                 |
input  bool    InpFridayCloseTrades   = false      ;  // FridayCloseTrades
input  string  InpFridayCloseTimeGMT  = "21:55"    ;  // FridayCloseTimeGMT
//+------------------------------------------------------------------+
//| Expert initialization function                                   |
//+------------------------------------------------------------------+
int OnInit()
  {
//--- initialize common information
   m_trade.SetAsyncMode(true);
   m_trade.SetDeviationInPoints(INT_MAX);
   m_trade.SetMarginMode();
   m_trade.LogLevel(LOG_LEVEL_ERRORS);
//--- Initialization completed successfully
   return(INIT_SUCCEEDED);
  }
//+------------------------------------------------------------------+
//| Expert tick function                                             |
//+------------------------------------------------------------------+
void OnTick()
  {
   if(InpFridayCloseTrades && PositionsTotal() > 0 && FridayTimeIsActive())
     {
      CloseAllPositions();
     }
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
bool FridayTimeIsActive()
  {
   MqlDateTime dt;

   datetime gmtTime = TimeGMT(dt);
   datetime endTime = StringToTime(TimeToString(gmtTime,TIME_DATE) + " " + InpFridayCloseTimeGMT);

   if(dt.day_of_week==(ENUM_DAY_OF_WEEK)FRIDAY && gmtTime >= endTime)
     {
      return(true);
     }

   return(false);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void CloseAllPositions()
  {
//--- close market positions
   for(int i=PositionsTotal()-1; i>=0 && !IsStopped(); i--)
     {
      if(m_position.SelectByIndex(i))
        {
         //--- trading object
         m_trade.SetExpertMagicNumber(m_position.Magic());
         m_trade.SetTypeFillingBySymbol(m_position.Symbol());
         //--- close positions
         if(m_trade.PositionClose(m_position.Ticket()) && (m_trade.ResultRetcode()==TRADE_RETCODE_DONE || m_trade.ResultRetcode()==TRADE_RETCODE_PLACED))
            PrintFormat("Position #%I64u on %s to be closed",m_position.Ticket(),m_position.Symbol());
         else
            PrintFormat("> Error closing position #%I64u on %s (%s)",m_position.Ticket(),m_position.Symbol(),m_trade.ResultComment());
        }
      }
   }
//+------------------------------------------------------------------+
